#-------------------------------------------------------#
#-------------------------------------------------------#
#-------------------------------------------------------#
#-------------------------------------------------------#
#  IF YOU WANT TO INCLUDE YOUR OWN DATA YOU WILL NEED:  #
#                                                       #
#  1) Store your data in the directory: dir_data        # 
#     as a text file ->  your_data.txt                  #
#                                                       #
#  2) Create a folder to store the MCMC output and      #
#     further  outputs, the folder should be in the     #
#     directory: dir                                    #
#                                                       #
#  3) Modify the functions:                             #
#                                                       #
#    1  - Function Loading Data, see type == 9.         #
#    8  - Reference Parameters, see type == 9.          #
#    11 - Plotting the Predictive, see '9'.             #
#    13 - Plotting the Scaled Components Predictive     #
#         and Single Best Clustering, see '9'.          #
#-------------------------------------------------------#
#-------------------------------------------------------#
#-------------------------------------------------------#
#-------------------------------------------------------#
#           1- Function Loading Data                    #
#-------------------------------------------------------#
#                                                       #
#    This function loads a data set with the structure  #
#           needed to implement our programs            #
#                                                       #
#             Input Data                                #
#                                                       #
# type   - 1-> data from model (4.1) pp 322 from a)     # 
#          2-> data from model (4.2) pp 322 from a)     #
#              and Model 3 from d)                      #
#          3-> the famous galaxy data                   #
#          4-> the acidity data from c)                 # 
#          5-> the enzyme data from c)                  #
#          6-> the famous crab data generated           #
#              following b)                             #
#          7-> data from Model 1, a version of the      #  
#              crab data from d)                        #
#          8-> data from Model 2 from d)                #
#          9-> YOUR OWN DATA                            #
#                                                       #
# n_star   - length of the grid for the predictive      #
# dir_data - directory where all the data sets are,     #
#            all are stored in .txt files, e.g.         #
#                                                       #
#    /home/username/Documents/LabelSwitching/Data/      #
#                                                       #
# dir     - directory where the MCMC output and         #
#           further results will be stored, e.g.        #
#                                                       #
#    /home/username/Documents/LabelSwitching/Results/   #
#                                                       #
# a) Papastamoulis, P., and Iliopoulos, G. (2010)       #
# b) Pearson, K.(1894)                                  #
# c) Richardson, S., and Green, P. J.(1997)             #
# d) Rodriguez, C. E. and Walker, S. G. (2012)          # 
#                                                       #
#              Output Data                              #
#                                                       #
# A list where                                          #
#  $type    - One of the above options                  # 
#  $k       - number of components                      # 
#  $y       - the univariate data to analyze            #
#  $y_star  - the grid to generate the predictive       #
#  $dir_res - the directory where the results will be   #
#             stored, the results for each data set     # 
#             are stored in a different folder          #
#             you need to create this folder first.     #
#             To analyze 1-8 the folders mix1, mix2,    #
#             galaxy, acidity, enzyme, crab, mix3 and   # 
#             mixext should be already in               #
#                                                       #
#    /home/username/Documents/LabelSwitching/Results/   #
#                                                       #
#-------------------------------------------------------#
input_output_load <- function(type, n_star, dir_data, dir){	
	if(type == 1){
		k <- 3
		y <- as.matrix(read.table(paste(dir_data, 'mix1.txt' ,sep=''),
															header = FALSE, sep = ' ', col.names = 'Data', quote = "")) # reading the data
		y_star <- seq(-25, 27, length.out= n_star)  # grid for the predictive
		dir_res <- paste(dir, 'mix1/', sep='') # directory to store results
	}
	if(type == 2){
		k <- 5 
		y <- as.matrix(read.table(paste(dir_data, 'mix2.txt' ,sep=''),
															header = FALSE, sep = ' ', col.names = 'Data', quote = ""))
		y_star <- seq(9, 40, length.out= n_star)  
		dir_res <- paste(dir, 'mix2/', sep='') 
	}
	if(type == 3){
		k <- 6
		y <- as.matrix(read.table(paste(dir_data, 'galaxy.txt' ,sep=''),
															header = FALSE, sep = ' ', col.names = 'Data', quote = ""))	
		y_star <- seq(7, 37, length.out= n_star) 
		dir_res <- paste(dir, 'galaxy/', sep='') 
	}
	if(type == 4){
		k <- 4
		y <- as.matrix(read.table(paste(dir_data, 'acidity.txt' ,sep=''),
															header = FALSE, sep = ' ', col.names = 'Data', quote = ""))
		y_star <- seq(2, 8, length.out= n_star)   
		dir_res <- paste(dir, 'acidity/', sep='') 
	}
	if(type == 5){
		k <- 3
		y <- as.matrix(read.table(paste(dir_data, 'enzyme.txt' ,sep=''),
															header = FALSE, sep = ' ', col.names = 'Data', quote = ""))
		y_star <- seq(-.1, 3.2, length.out= n_star)  
		dir_res <- paste(dir, 'enzyme/', sep='') 
	}
	if(type == 6){
		k <- 2
		y <- as.matrix(read.table(paste(dir_data, 'mycrab.txt' ,sep=''),
															header = FALSE, sep = ' ', col.names = 'Data', quote = ""))
		y_star <- seq(.57, .7, length.out= n_star)  
		dir_res <- paste(dir, 'crab/', sep='') 
	}
	if(type == 7){
		k <- 2
		y <- as.matrix(read.table(paste(dir_data, 'mix3.txt' ,sep=''),
															header = FALSE, sep = ' ', col.names = 'Data', quote = ""))
		y_star <- seq(.57, .7, length.out= n_star)  
		dir_res <- paste(dir, 'mix3/', sep='') 
	}
	if(type == 8){
		k <- 4
		y <- as.matrix(read.table(paste(dir_data, 'mixext.txt' ,sep=''),
															header = FALSE, sep = ' ', col.names = 'Data', quote = ""))
		y_star <- seq(-7, 7, length.out= n_star)  
		dir_res <- paste(dir, 'mixext/', sep='') 
	}
#	if(type == 9){ # To use your own data, uncomment and fill in as adequate
#   k <- the number of components, known or assumed  
#		y <- as.matrix(read.table(paste(dir_data, 'your_data.txt' ,sep=''),
#															header = FALSE, sep = ' ', col.names = 'Data', 
#															quote = ""))    # Store your data in dir_data as a .txt file
#                                             # Test it, run the function and plot a histogram: hist(in_out$y) 
#                                             # to check that it has been stored adequately and that this 
#                                             # function is reading it correctly   
#   lower_bound <- min(y) - something #  define the range to calculate the predictive
#   upper_bound <- max(y) + something 
#		y_star <- seq(lower_bound, upper_bound, length.out= n_star)  # grid for the predictive
#		dir_res <- paste(dir, 'your_folder/', sep='') # directory to store the MCMC output (create your own folder)
#	}
	list(type = type, k = k, y = y, y_star = y_star, dir_res = dir_res)
}
#-------------------------------------------------------#
#           2- Storage Requirements                     #
#-------------------------------------------------------#
#                                                       #
#  All the output from the Gibbs is stored directly     #
#  in text files, this function returns the approximate #
#  text file sizes for each one of this files.          #
#  In particular the text file size for the class probs # 
#  and allocations are exact.                           #  
#                                                       #
#             Input Data                                #
#                                                       #
# m        - number of iterations to store              #
# k        - number of components                       #
# y        - data (univariate)                          #
#                                                       #
#              Output Data                              #
#                                                       #
# A data frame with the text file sizes for             #
# w.txt        - matrix of mxk of weights               #
# mu.txt       - matrix of mxk of means                 #
# sigma2.txt   - matrix of mxk of standard deviations   #
# pz.txt       - m matrices of nxk of class probs       #
# z.txt        - m vectors of 1xn of latent allocations #
# log_post.txt - m log posteriors up to a constant      #
# Total        - Total text file size                   #
#-------------------------------------------------------#
storage_requirements <- function(m, k, y){
	n <- length(y)
	oneMB <- 1024^2
	oneGB <- 1024^3
	Variable <- c('w',         'mu', 'sigma2',    'cp', 'alloc', 'log_post', 'Total')
	Unit <- c('MB', 'MB', 'MB', 'GB', 'MB',   'MB', 'GB') 
	Accuracy <- c('exact', 'approx', 'approx', 'exact', 'exact',   'approx', 'approx') 
	n1 <- nchar(as.character(trunc(mean(y)))) + 1
	n2 <- nchar(as.character(trunc(var(y)))) + 1
	guess <- 6
	Memory <- rep(NA, 7)
	Memory[1] <- (20*k + 1)*m 
	Memory[2] <- (n1 + 20*k + 1)*m 
	Memory[3] <- (n2 + 20*k + 1)*m 
	Memory[4] <- n*(20*k + 1)*m
	Memory[5] <- (3*n + 1)*m
	Memory[6] <- (guess + 21)*m
	Memory[7]  <- sum(Memory[1:6])/oneGB
	Memory[1] <- Memory[1]/oneMB 
	Memory[2] <- Memory[2]/oneMB 
	Memory[3] <- Memory[3]/oneMB
	Memory[4] <- Memory[4]/oneGB
	Memory[5] <- Memory[5]/oneMB
	Memory[6] <- Memory[6]/oneMB
	Memory <- round(Memory, 2)
	TFS <- data.frame(Variable, Memory, Unit, Accuracy)
	TFS
}
#-------------------------------------------------------#
#             3- Function Standard Gibbs                #
#-------------------------------------------------------#
#                                                       #
# This function runs a Gibbs sampler to fit a Mixture   #
#   of univ Normals with a known number of components.  #
# We implemented the version of:                        #
#         Richardson, S., and Green, P. (1997)          #
#                                                       #
#             Input Data                                #
#                                                       #
# burn     - number of iterations to burn               #
# m        - number of iterations to store              #
# k        - number of components                       #
# y        - data (univariate)                          #
# delta    - prior for the weights                      # 
# mu0      - prior mean for the means                   #
# kappa    - prior precision for the means              #
# alpha    - shape parameter for tau = 1/sigma2         # 
# gg       - Hyper prior for beta E(beta) = gg/hh       # 
# hh       - Hyper prior for beta                       # 
# dir      - directory to store the MCMC output         #
#                                                       #
#             Output Data                               #
#                                                       #
#   The following files are stored in                   #
#   the directory: dir (NOT an output of R):            #
# w.txt        - matrix of mxk of weights               #
# mu.txt       - matrix of mxk of means                 #
# sigma2.txt    - matrix of mxk of standard deviations   #
# pz.txt       - m matrices of nxk of class probs       #
# z.txt        - m vectors of 1xn of latent allocations #
# log_post.txt - m log posteriors up to a constant      #
#-------------------------------------------------------#
normal_gibbs <- function(burn, m, k, y, delta, mu0, kappa, alpha, gg, hh, dir){
  dir_z <- paste(dir, 'z.txt', sep ='')
  dir_pz <- paste(dir, 'pz.txt', sep ='')
  dir_w <- paste(dir, 'w.txt', sep ='')
  dir_mu <- paste(dir, 'mu.txt', sep ='')
  dir_sigma2 <- paste(dir, 'sigma2.txt', sep ='')
  dir_log_post <- paste(dir, 'log_post.txt', sep ='')
  n <- length(y)
  .C("main_normal", as.integer(burn), as.integer(m), as.integer(k), as.integer(n), as.double(y),
     as.double(delta), as.double(mu0), as.double(kappa), as.double(alpha), as.double(gg),
     as.double(hh), as.character(dir_z), as.character(dir_pz), as.character(dir_w),
     as.character(dir_mu), as.character(dir_sigma2), as.character(dir_log_post))[[3]]
}
#------------------------------------------------------------#
#   4-  Function Representatives for the Data Algorithm      #
#------------------------------------------------------------#
#                                                            #
#       This function finds representatives for the Data     #
#     strategy as in Algorithm 4 in the paper                #
#                                                            #
#                   Input Data                               #
#                                                            #
# k      - number of components                              #
# m      - number of iterations stored during the MCMC       #
# y      - data (univariate)                                 #
# dir    - directory where the MCMC output should be         #
#                                                            #
#                   Output Data                              #
#                                                            #
#   A list with the representatives for the means and        #
#   variances:                                               #
#    $mu   - representatives for the means                   # 
#    $sig2 - representatives for the variances               #
#------------------------------------------------------------#
representatives_mean_var <- function(k, m, y, dir){
  n <- length(y)
	y0 <- min(y)
	yn <- max(y) 
	R <- yn - y0
	mu <- y0 + R*(seq(1, k, by = 1)/(k + 1))
	sigma2 <- rep(R/k, k)^2
  dir_z <- paste(dir, 'z.txt', sep ='')
  res <- .C("group_mean_var", as.integer(k), as.integer(n), as.integer(m), as.double(y), 
            as.double(mu), as.double(sigma2), as.character(dir_z))
	list(mu = res[[5]], sig2 = res[[6]])
}
#------------------------------------------------------------#
#         5-  Function MAP estimate for d                    #
#------------------------------------------------------------#
#                                                            #
# This function finds the MAP estimate needed by the the ECR #
#                      algorithm                             #
#                                                            #
#                   Input Data                               #
#                                                            #
# k      - number of components                              #
# y      - data (univariate)                                 #
# dir    - directory where the MCMC output should be stored  #
#                                                            #
#                   Output Data                              #
#                                                            #
# z_map  - the MAP estimate for d                            #
#------------------------------------------------------------#
find_z_MAP <- function(k, y, dir){
  n <- length(y)
	# directories where log_post and z have been stored #
	dir_log_post <- paste(dir, 'log_post.txt', sep ='')
	dir_z <- paste(dir, 'z.txt', sep ='')
	# finding the z_MAP estimate among the 1000 highest log post values 
	# we want a z_map that allocates observations to k groups!
	log_post <- as.matrix(read.table(file = dir_log_post, header = FALSE))
	o <- order(log_post, decreasing = T)
	I <- rep(1, n)
	bandera <- 0
	for(l in 1:1000){
		z_map <- scan(dir_z, skip = o[l] - 1, nlines = 1, quiet = T)
		if(length(z_map) < n){
			stop("Problem reading z_MAP")
		}
		nj <- tapply(I, z_map, sum)
		if(length(nj) == k){	
			bandera <- 1
			break
		}		
	}
	if(bandera == 0){
		stop("No observations assigned to a component")		
	}
	z_map	
}
#------------------------------------------------------------#
#       6-  Interface to the relabeling algorithms           #
#------------------------------------------------------------#
#                                                            #
#    This function calls one of the three relabeling         #
#     algorithms: Data, ECR or KL as described in            #
#      Rodriguez, C. E. and Walker, S. G. (2012)             # 
#         Algorithms 5, 2 and 1 respectively                 #
#                                                            #
#                   Input Data                               #
#                                                            #
# option - one of the strategies : 'data', 'ECR' or 'KL'     #
# k      - number of components                              #
# m      - number of iterations stored during the MCMC       #
# y      - data                                              #
# mu     - vector of 1xk representatives for the means       #
# sigma2 - vector of 1xk representatives for the variances   # 
# z_map  - the MAP estimator for d                           #
# rnd    - if 0 the identity permutation will be used as     #
#          starting point for KL otherwise it will be random #
# epsilon- accuracy to detect a fix point, KL will stop if   #
#                |sum min L^t - sum min L^{t-1}| < epsilon   #
#          where sum min L^t is as in Algorithm 1, see pp. 9 #
# cont   - maximum number of iterations for the KL algorithm #
# prob_name - if KL is selected then this name will be       # 
#             given to the file where the posterior class    #
#             probs will be stored, it should be of the      #
#             form: 'class_probs.txt'                        # 
# dir    - directory where the MCMC output should be stored  #
# label_name  - the name that will be given to the file      #
#               where the relabeling will be stored, it      #
#               should be of the form:  'label_name.txt'     #
#                                                            # 
#  Note that if for example option = 'data' then z_map, rnd, # 
#  epsilon, cont and prob_name will be ignored (no needed)   #
#                                                            #
#                  Output Data                               #
#                                                            #
#  - A matrix of mxk of permutations will be stored          #
#    in the file 'label_name.txt' in the directory: dir      #
#       (NOT an output of R):                                #
#  - If the option KL is chosen the posterior classification #
#    probabilities will be stored as 'probs_name.txt' in the #
#    directory: dir  (NOT an output of R):                   #
#------------------------------------------------------------#
relabel <- function(option = 'Data', k, m, y, mu = NA, sigma2 = NA, z_map = NA, rnd = 0, epsilon = 0, 
                    cont = 0, prob_name = NULL, dir, label_name){
	n <- length(y) 
	if(option == 'Data' & length(mu)==k){
		tt <- system.time(data_relabel(k, n, m, y, mu, sigma2, dir, label_name))
	}else{
		if(option == 'ECR' & length(z_map)==n){			
			tt <- system.time(ECR_relabel(k, n, m, z_map, dir, label_name))
		}else{
			if(option == 'KL'){
				if(as.integer(cont) > 0){
					if(epsilon > 0){
						if(is.character(prob_name)){
							ttN <- system.time(KL_ini_label(k, m, rnd, dir, label_name))		
							tt <- system.time(KL_relabel(k, n, m, epsilon, cont, dir, label_name, prob_name))
						}else{
							print('Provide a name for the classification probabilities: prob_name')
						}
					}else{
						print('Provide degree of accuracy: epsilon')
					}							
				}else{
					print('Provide a maximum number of iterations: cont')
				}
			}else{
				print('Not a valid option')
			}
		}
	}
	tt[3]	
}
#------------------------------------------------------------#
#            Direct access to the Data algorithm             #
#------------------------------------------------------------#
#                                                            #
#                   Input Data                               #
#                                                            #
# k      - number of components                              #
# n      - number of observations -> length(y)               #
# m      - number of iterations stored during the MCMC       #
# mu     - vector of 1xk representatives for the means       #
# sigma2 - vector of 1xk representatives for the variances   # 
# dir    - directory where the MCMC output should be stored  #
# label_name  - the name that will be given to the file      #
#               where the relabeling will be stored, it      #
#               should be of the form:  'label_name.txt'     # 
#                                                            #
#                   Output Data                              #
#                                                            #
#   - A matrix of mxk of permutations will be stored         #
#     in the file 'label_name.txt' in the directory: dir     #
#       (NOT an output of R):                                #
#------------------------------------------------------------#
data_relabel <- function(k, n, m, y, mu, sigma2, dir, label_name){
  dir_z <- paste(dir, 'z.txt', sep ='')
  dir_label <- paste(dir, label_name, sep ='')
  .C("data_relabel", as.integer(k), as.integer(n), as.integer(m), as.double(y),
     as.double(mu), as.double(sigma2), as.character(dir_z), as.character(dir_label))[[1]]
}
#------------------------------------------------------------#
#  Direct access to the function that generates an initial   #
#  relabeling for the KL algorithm                           #
#------------------------------------------------------------#
#                                                            #
#                   Input Data                               #
#                                                            #
# k      - number of components                              #
# m      - number of iterations stored during the MCMC       #
# rnd    - if 0 the identity permutation will be used as     #
#          starting point for KL otherwise it will be random #
# dir    - directory where the MCMC output should be stored  #
# label_name  - the name that will be given to the file      #
#               where the relabeling will be stored, it      #
#               should be of the form:  'label_name.txt'     # 
#                                                            #
#                   Output Data                              #
#                                                            #
#   - A matrix of mxk of permutations will be stored         #
#     in the file 'label_name.txt' in the directory: dir     #
#       (NOT an output of R):                                #
#------------------------------------------------------------#
KL_ini_label <- function(k, m, rnd, dir, label_name){
  dir_label <- paste(dir, label_name, sep ='')
  .C("KL_initial_label", as.integer(k), as.integer(m), as.integer(rnd),
     as.character(dir_label))[[1]]
}
#------------------------------------------------------------#
#            Direct access to the KL algorithm               #
#------------------------------------------------------------#
#                                                            #
#                     IMPORTANT!!!!!!!!!                     #
#   The function KL_ini_label must be executed first!        #
#                                                            #
#                   Input Data                               #
#                                                            #
# k      - number of components                              #
# n      - number of observations -> length(y)               #
# m      - number of iterations stored during the MCMC       #
# epsilon- accuracy to detect a fix point, KL will stop if   #
#                |sum min L^t - sum min L^{t-1}| < epsilon   #
#          where sum min L^t is as in Algorithm 1, see pp. 9 #
# cont   - maximum number of iterations for the KL algorithm #
# prob_name - this name will be given to the file where the  #
#             post class probs will be stored, it should be  #
#             of the form: 'class_probs.txt'                 # 
# dir    - directory where the MCMC output should be stored  #
# label_name  - the name that will be given to the file      #
#               where the relabeling will be stored, it      #
#               should be of the form:  'label_name.txt'     # 
#               it should be the same as for the function:   #
#                          ---KL_ini_label---                #
#                                                            #
#                   Output Data                              #
#                                                            #
#   - A matrix of mxk of permutations will be stored         #
#     in the file 'label_name.txt' in the directory: dir     #
#       (NOT an output of R):                                #
#  - The posterior classification probabilities will         #
#     be stored as 'probs_name.txt' in the directory:        #
#    dir  (NOT an output of R):                              #
#------------------------------------------------------------#
KL_relabel <- function(k, n, m, epsilon, cont, dir, label_name, prob_name){
  dir_label <- paste(dir, label_name, sep ='')
  dir_pz <- paste(dir, 'pz.txt', sep ='')
  dir_pz_fin <- paste(dir, prob_name, sep ='')
  .C("KL_relabel", as.integer(k), as.integer(n), as.integer(m), as.double(epsilon),
     as.integer(cont), as.character(dir_label), as.character(dir_pz), 
     as.character(dir_pz_fin))[[1]]
}
#------------------------------------------------------------#
#            Direct access to the ECR algorithm              #
#------------------------------------------------------------#
#                                                            #
#                   Input Data                               #
#                                                            #
# k      - number of components                              #
# n      - number of observations -> length(y)               #
# m      - number of iterations stored during the MCMC       #
# z_map  - the MAP estimator for d                           #
# dir    - directory where the MCMC output should be stored  #
# label_name  - the name that will be given to the file      #
#               where the relabeling will be stored, it      #
#               should be of the form:  'label_name.txt'     # 
#                                                            #
#                   Output Data                              #
#                                                            #
#   - A matrix of mxk of permutations will be stored         #
#     in the file 'label_name.txt' in the directory: dir     #
#       (NOT an output of R):                                #
#------------------------------------------------------------#
ECR_relabel <- function(k, n, m, z_map, dir, label_name){  
  dir_z <- paste(dir, 'z.txt', sep ='')
  dir_label <- paste(dir, label_name, sep ='')
  .C("ECR_relabel", as.integer(k), as.integer(n), as.integer(m), as.integer(z_map),
     as.character(dir_z), as.character(dir_label))[[1]]
}
#------------------------------------------------------------#
#           7-  Posterior Mean Estimates                     #
#------------------------------------------------------------#
#          This function calculates the                      # 
#     Posterior Mean Estimates for a mixture of              #
#              normals with known k                          #
#                                                            # 
#                IMPORTANT!!!!!!!!!                          #
#    The relabeling used to recover identifiability must be  #
#                     generated first                        #
#                                                            #
#                Input Data                                  #
#                                                            #
# k          - number of components                          #
# m          - number of iterations stored during the MCMC   #
# dir        - directory where the MCMC output and           #
#              relabeling should be stored.                  #
# label_name - the name that was given to the relabeling     #
#                                                            #
#                Output Data                                 #
#                                                            #
#  A list such that:                                         #
#    $w    - are the posterior means for the weights         #
#    $mu   - are the posterior means for the means           #
#    $sig2 - are the posterior means for the variances       #
#------------------------------------------------------------#
posterior_mean_estimates <- function(k, m, dir, label_name){
	w <- rep(0, k)
	mu <- rep(0, k)
	sigma2 <- rep(0, k)
  dir_label <- paste(dir, label_name, sep ='')
  dir_w <- paste(dir, 'w.txt', sep ='')
  dir_mu <- paste(dir, 'mu.txt', sep ='')
  dir_sigma2 <- paste(dir, 'sigma2.txt', sep ='')
  res <- .C("posterior_mean", as.integer(k), as.integer(m), as.double(w), as.double(mu), as.double(sigma2),
						 as.character(dir_label), as.character(dir_w), as.character(dir_mu), as.character(dir_sigma2))
	list(w = res[[3]], mu = res[[4]], sig2 = res[[5]])
}
#----------------------------------------------------------#
#             8 - Reference Parameters                     #
#----------------------------------------------------------#
#                                                          #
#  This function gives reference or true parameters to     #
#  compare and measure the relative errors between the     #
#  posterior mean estimates and the reference values.      #
#  This is needed if we want to perform a comparison       #
#  between the accuracy of posterior mean estimates        #
#  achieved with each relabeling strategy.                 #
#                                                          #
#             Input Data                                   #
#                                                          #
#   in_out$type, the selected data set to analyze          #
#                                                          #
#              Output Data                                 #
#                                                          #
#  A list such that:                                       #
#   $w    - true or reference parameters for the weights   #
#   $mu   - true or reference parameters for the means     #
#   $sig2 - true or reference parameters for the variances #
#----------------------------------------------------------#
ref_values <- function(type){ 
	if(type == 1){ # mix1, true values
		w <- c(0.1, 0.65, 0.25)
		mu <- c(-20.0, 20.0, 21.0)
		sig2 <- c(1.0, 3.0, 0.5)
	}
	if(type == 2){ # mix2, true values
		w <- c(0.2, 0.2, 0.25, 0.2, 0.15)
		mu <- c(19.0, 19.0, 23.0, 29.0, 33.0)
		sig2 <- c(5.0, 1.0, 1.0, 0.5, 2.0)
	}
	if(type == 3){ # galaxy data, just reference values#
		w <- c(0.09, 0.06, 0.285, 0.41, 0.111, 0.044)
		mu <- c(9.71, 18.71, 19.87, 22.3, 23.49, 32.82)
		sig2 <- c(.57, 1.78, .755, 3.54, 1.9, 1.75)
	}
	if(type == 4){ # enzyme data, just reference values#
		w <- c(0.296, 0.281, 0.273, 0.150)
		mu <- c(4.25, 4.61, 6.43, 5.37)
		sig2 <- c(0.125, 0.252, 0.167, 0.3116)
	}
	if(type == 5){ # acidity data, just reference values#
		w <- c(0.606, 0.25, 0.144)
		mu <- c(0.19, 1.44, 1.29)
		sig2 <- c(.0068, 0.238, 0.0558)
	}
	if(type == 6){ # crab data, reference values#
		w <- c(0.4, 0.6)
		mu <- c(0.63, 0.65)
		sig2 <- c(0.00032, 0.00016)
	}
	if(type == 7){ # simulated crab data, true values #
		w <- c(0.4, 0.6)
		mu <- c(0.63, 0.65)
		sig2 <- c(0.00032, 0.00016)
	}
	if(type == 8){ # mixext, true values
		w <- c(0.25, 0.25, 0.25, 0.26)
		mu <- c(-3, -1, 1, 3)
		sig2 <- c(1, 1, 1, 1)
	}
	#if(type == 9){ # include the true or reference parameters of your model
		#w <- c()
		#mu <- c()
		#sig2 <- c()
	#}
	list(w = w, mu = mu, sig2 = sig2)
}
#------------------------------------------------------------#
#       9 - Comparison of Posterior Mean Estimates           #
#------------------------------------------------------------#
#          This function calculates the                      # 
#   posterior mean estimates for a mixture of                #
#   normals with known k for the three relabelings:          #
#                  Data, ECR and KL                          #
#  (in the same loop) and it displays the results with the   #
#       aim of to compare the algorithms                     #
#                                                            # 
#                IMPORTANT!!!!!!!!!                          #
#    The three relabelings must be generated first           #
#                                                            #
#                Input Data                                  #
#                                                            #
# k          - number of components                          #
# m          - number of iterations stored during the MCMC   #
# dir        - directory where the MCMC output and           #
#              relabeling should be stored.                  #
# ref        - list of reference values for the weights      #
#              means and var: ref$w, ref$mu and ref$sig2.    #
#              See the function  8 - Reference Parameters    #
# label_Data - name that was given to the Data relabeling    #
# label_ECR  - name that was given to the ECR relabeling     #
# label_KL   - name that was given to the KL relabeling      #
#                                                            #
#                Output Data                                 #
#                                                            #
#  -A list such that:                                        # 
#    $W      - comparison for he weights                     #
#    $MU     - comparison for the means                      #
#    $SIGMA2 - comparison for the variances                  #   
#                                                            #
#  -A matrix of permutations such that if the indexes of the #
#   posterior mean estimates (PME) are permuted according to #
#   it, the total relative error between the posterior mean  #
#   estimates and reference values is minimum.               # 
#   It will be stored in 'perm.txt' in the directory: dir    #
#   (NOT an output of R):                                    #
#   First row -  permutation for the PME from Data strategy  #  
#   Second row-  permutation for the PME from ECR strategy   #
#   Third row -  permutation for the PME from KL strategy    #  
#------------------------------------------------------------#
compar_posterior_mean_estimates <- function(k, m, dir, ref, label_Data, label_ECR, label_KL){
	w <- rep(0, 3*k)
	mu <- rep(0, 3*k)
	sigma2 <- rep(0, 3*k)
  dir_label1 <- paste(dir, label_Data, sep ='')
	dir_label2 <- paste(dir, label_ECR, sep ='')
	dir_label3 <- paste(dir, label_KL, sep ='')
  dir_w <- paste(dir, 'w.txt', sep ='')
  dir_mu <- paste(dir, 'mu.txt', sep ='')
  dir_sigma2 <- paste(dir, 'sigma2.txt', sep ='')
  res <- .C("compar_posterior_mean", as.integer(k), as.integer(m), as.double(w), as.double(mu), as.double(sigma2),
						 as.character(dir_label1), as.character(dir_label2), as.character(dir_label3), as.character(dir_w), as.character(dir_mu), as.character(dir_sigma2))
	w_Data <- res[[3]][1:k]
	w_ECR <- res[[3]][1:k + k]
	w_KL <- res[[3]][1:k + 2*k]
	mu_Data <- res[[4]][1:k]
	mu_ECR <- res[[4]][1:k + k]
	mu_KL <- res[[4]][1:k + 2*k]
	sig2_Data <- res[[5]][1:k]
	sig2_ECR <- res[[5]][1:k + k]
	sig2_KL <- res[[5]][1:k + 2*k]
	lab1 <- find_perm_min_tot_rel_error(k, ref$w, ref$mu, ref$sig2, w_Data, mu_Data, sig2_Data)
	lab2 <- find_perm_min_tot_rel_error(k, ref$w, ref$mu, ref$sig2, w_ECR, mu_ECR, sig2_ECR)
	lab3 <- find_perm_min_tot_rel_error(k, ref$w, ref$mu, ref$sig2, w_KL, mu_KL, sig2_KL)
	e_w_Data <- sum(abs((ref$w - w_Data[lab1])/ref$w))
	e_w_ECR <- sum(abs((ref$w - w_ECR[lab2])/ref$w))
	e_w_KL <- sum(abs((ref$w - w_KL[lab3])/ref$w))
	e_mu_Data <- sum(abs((ref$mu - mu_Data[lab1])/ref$mu))
	e_mu_ECR <- sum(abs((ref$mu - mu_ECR[lab2])/ref$mu))
	e_mu_KL <- sum(abs((ref$mu - mu_KL[lab3])/ref$mu))
	e_sig2_Data <- sum(abs((ref$sig2 - sig2_Data[lab1])/ref$sig2))	
	e_sig2_ECR <- sum(abs((ref$sig2 - sig2_ECR[lab2])/ref$sig2))	
	e_sig2_KL <- sum(abs((ref$sig2 - sig2_KL[lab3])/ref$sig2))
	A1 <- rbind(0, e_w_Data, e_w_ECR, e_w_KL)
	A2 <- rbind(0, e_mu_Data, e_mu_ECR, e_mu_KL)
	A3 <- rbind(0, e_sig2_Data, e_sig2_ECR, e_sig2_KL)
	W_summ <- cbind(rbind(ref$w, w_Data[lab1], w_ECR[lab2], w_KL[lab3]), A1)
	MU_summ <- cbind(rbind(ref$mu, mu_Data[lab1], mu_ECR[lab2], mu_KL[lab3]), A2)
	S2_summ <- cbind(rbind(ref$sig2, sig2_Data[lab1], sig2_ECR[lab2], sig2_KL[lab3]), A3)
	rownames(W_summ) <- c('True', 'Data', 'ECR', 'KL')
	colnames(W_summ) <- c(paste('w', 1:k, sep=''), 'Rel.Err')
	rownames(MU_summ) <- c('True', 'Data', 'ECR', 'KL')
	colnames(MU_summ) <- c(paste('mu', 1:k, sep=''), 'Rel.Err')
	rownames(S2_summ) <- c('True', 'Data', 'ECR', 'KL')
	colnames(S2_summ) <- c(paste('sig2', 1:k, sep=''), 'Rel.Err')
	perm <- rbind(lab1, lab2, lab3) 
	write.table(perm, file = paste(dir, 'perm.txt' ,sep=''), append = FALSE, sep = ' ',
							eol = "\n", row.names = F, col.names = F)
	list(W = W_summ, MU = MU_summ, SIGMA2 = S2_summ)
}
#------------------------------------------------------------#
#          10 -  Scaled Components Predictive                #
#------------------------------------------------------------#
#          This function calculates the                      # 
#   scaled components predictive for a mixture of            #
#              normals with known k                          #
#                                                            # 
#                IMPORTANT!!!!!!!!!                          #
#    The relabeling used must be generated first             #
#                                                            #
#                Input Data                                  #
#                                                            #
# k          - number of components                          #
# m          - number of iterations stored during the MCMC   #
# y_star     - grid to calculate the predictive              #
# dir        - directory where the MCMC output and           #
#              relabeling should be stored.                  #
# label_name - the name that was given to the relabeling     #
# pred_name  - the name that will be given to the file where #
#              the predictive will be stored, it should be   #  
#              of the form: 'pred_name.txt'                  # 
#                                                            #
#                Output Data                                 #
#                                                            #
#  The predictive is stored in 'pred_name.txt'               # 
#  in the directory: dir  (NOT an output of R):              #
#------------------------------------------------------------#
scaled_comp_pred <- function(k, m, y_star, dir, label_name, pred_name){
  n_star <- length(y_star)
  dir_label <- paste(dir, label_name, sep ='')
  dir_w <- paste(dir, 'w.txt', sep ='')
  dir_mu <- paste(dir, 'mu.txt', sep ='')
  dir_sigma2 <- paste(dir, 'sigma2.txt', sep ='')
  dir_pred <- paste(dir, pred_name, sep ='')
  .C("comp_pred", as.integer(k), as.integer(m), as.integer(n_star), as.double(y_star),
     as.character(dir_label), as.character(dir_w), as.character(dir_mu), as.character(dir_sigma2),
     as.character(dir_pred))[[1]]
}
#------------------------------------------------------------#
#            11-  Plotting the Predictive                    #
#------------------------------------------------------------#
#                   IMPORTANT!!!!!!!!!                       #
#   The function that generates the scaled components        #
#   predictive must be executed before this function         #
#                                                            #
#             Input Data                                     #
#  in_out - a list from the Function Loading Data            #
#   in_out$type, the chosen data set to analyze              #
#  	in_out$y, the univariate data to analyze                 #
#  	in_out$y_star, the grid to generate the predictive       #
#  	in_out$dir_res, directory where the MCMC output and      # 
#                   relabeling should be stored              #
# pred_name  - the name that was given to the file to store  #
#              the scaled components predictive. It does not #
#              matter the relabeling used, the predictive is #
#              invariant to permutations of the components.  #
#                                                            #
#              Output Data                                   #
#                                                            #
#  A graphic for the predictive:  p(y_star|y)                #
#------------------------------------------------------------#
my_pred_graph <- function(in_out, pred_name){
	r <- switch(as.character(in_out$type), 
              '1' = list(a = -25, b = 27,  d = 0.25, nc = 40), #mix1
							'2' = list(a = 10,  b = 40,  d = .12,  nc = 40), #mix2
						  '3' = list(a = 7,   b = 36,  d = .30,  nc = 40), #galaxy
              '4' = list(a = 2,   b = 8,   d = 1,    nc = 25), #acidity
              '5' = list(a = 0,   b = 3.5, d = 3.5,  nc = 30), #enzyme
							'6' = list(a = .57, b = .7,  d = 25,   nc = 40), #crab
              '7' = list(a = .57, b = .7,  d = 25,   nc = 40), #mix3
							'8' = list(a = -7,  b = 7,   d = .13,  nc = 20)) #mixext
              #'9' = list(a = ?,  b =? ,   d =? ,    nc = ?))  #yourdata
              # To display the predictive for your data:
              #    - Add a semicolon before the last parenthesis in '8'
              #    - Erase that last parenthesis and uncomment '9', and fill in as adequate.
							# Note: xlim <- c(a, b), ylim <- c(0, d), and nc the number of bins for the histogram 
	# Reading the Predictive #
	pred_comp <- as.matrix(read.table(file = paste(in_out$dir_res, pred_name, sep=''), header = FALSE)) 
	pred <- rowSums(pred_comp)
	hist(in_out$y, main= 'Predictive', xlab = '', ylab = 'Relative Frequency', xlim = c(r$a, r$b), 
       ylim = c(0, r$d), probability = T, axes = F, nclass = r$nc, col='gray80', border='white')
	axis(1, seq(r$a, r$b, length.out = 5), cex.axis = .8)
	axis(2, round(seq(0, r$d, length.out = 5), 3), cex.axis = .8)
	lines(in_out$y_star, pred, lwd = 2)
	box(lwd = 2, col = 'black')
}
#------------------------------------------------------------#
#     12 -  Posterior Classification Probabilities           #
#------------------------------------------------------------#
#           This function calculates the                     #  
#    posterior classification probabilities for a            #
#           mixture of normals with known k                  #  
#                                                            #
#    Only useful for Data or ECR relabelings:                #
#                KL relabeling gives this as an output.      #
#                                                            #
#                IMPORTANT!!!!!!!!!                          #
#    The relabeling used must be generated first             #
#                                                            #
#               Input Data                                   #
#                                                            #
# k          - number of components                          #
# m          - number of iterations stored during the MCMC   #
# y          - data (univariate)                             #
# dir        - directory where the MCMC output and           #
#              relabeling should be stored.                  #
# label_name - the name that was given to the relabeling     #
# probs_name - the name that will be given to the file where #
#              the post class probs will be stored, it should# 
#              be of the form: 'probs_name.txt'              # 
#                                                            #
#              Output Data                                   #
#                                                            #
#  The posterior classification probabilities will           #
#  be stored in 'probs_name.txt' in the directory:           #
#  dir  (NOT an output of R):                                #
#------------------------------------------------------------#
class <- function(k, m, y, dir, label_name, probs_name){
  n <- length(y)
  dir_label <- paste(dir, label_name, sep ='')
  dir_pz <- paste(dir, 'pz.txt', sep ='')
  dir_probs <- paste(dir, probs_name, sep ='')
  .C("classification", as.integer(k), as.integer(m), as.integer(n), as.double(y), as.character(dir_label),
     as.character(dir_pz), as.character(dir_probs))[[1]]
}
#----------------------------------------------------------#
#    13 - Plotting the Scaled Components Predictive        #
#               and Single Best Clustering                 #
#----------------------------------------------------------#
#                   IMPORTANT!!!!!!!!!                     #
#   The functions that generate the scaled components      #
#   predictive and the posterior classification probs      #
#   must be run before this function                       #
#                                                          #
#             Input Data                                   #
#                                                          #
#  k        - number of components                         #
#  in_out - a list from the Function Loading Data          #
#   in_out$type, the chosen data set to analyze            #
#  	in_out$y, the univariate data to analyze               #
#  	in_out$y_star, the grid to generate the predictive     #
#  	in_out$dir_res, - directory where the MCMC output      #
#                     and relabeling should be stored      #
# pred_name  - the name that was given to the file to      # 
#              store the scaled components predictive.     #
#              Here it does matter the relabeling used.    #
# prob_name  - the name that was given to the file to      # 
#              store the posterior classification probs.   #
#              Here it does matter the relabeling used.    #
# comparison - If this parameter is equal to:              #
#                   'Data', 'ECR' or 'KL'                  #
#              then the program will search for the file   #
#              perm.txt in the directory: dir, to order    #
#              the components according to a permutation.  #
#              This is only useful when comparing between  #
#              the different algorithms.                   #
#              For further details see the function:       #
#               9 - Comparison of Posterior Mean estimates #
#              if no comparison is needed give a different #
#              value.                                      #
#                                                          #
#  THE RELABELING USED TO GENERATE THE scaled components   # 
#  predictive AND posterior classification probs SHOULD BE #
#  THE SAME!                                               #
#                                                          #
#              Output Data                                 #
#                                                          #
#  A graphic for the scaled components predictive along    #
#  with the single best clustering generated with a given  #
#  relabeling                                              #
#----------------------------------------------------------#
my_scaled_clust_graph <- function(k, in_out, pred_name, prob_name, comparison){
	n <- length(in_out$y)
	single_best <- rep(NA, n)
	r <- switch(as.character(in_out$type), 
							'1' = list(a = -25, b = 27,  c = -.05, d = 0.25, e = -.003, nc = 40), #mix1
   						'2' = list(a = 10,  b = 40,  c = -.02, d = .12,  e = -.001, nc = 40), #mix2 
              '3' = list(a = 7,   b = 36,  c = -.05, d = .30,  e = -.008, nc = 40), #galaxy
              '4' = list(a = 2,   b = 8,   c = -.1,  d = 1,    e = -.02,  nc = 25), #acidity
              '5' = list(a = 0,   b = 3.5, c = -.3,  d = 3.5,  e= -.09,   nc = 30), #enzyme             
							'6' = list(a = .57, b = .7,  c = -2,   d = 25,   e = -.5,   nc = 40), #crab
              '7' = list(a = .57, b = .7,  c = -2,   d = 25,   e = -.5,   nc = 40), #mix3
							'8' = list(a = -7,  b = 7,   c = -.02, d = .13,  e = -.001, nc = 20)) #mixext
					    #'9' = list(a = ?,  b =? ,   c= ?,     d?,       e =? ,     nc = ?))  #yourdata
              # To display the scaled components predictive and single best clust for your data:
              #    - Add a semicolon before the last parenthesis in '8'
              #    - Erase that last parenthesis and uncomment '9', and fill in as adequate.
							#  Note:  xlim <- c(a, b), ylim <- c(c, d), c -> gives space below 0 to plot 
              #         the single best clust (sbc)
							#         u ~ U(c, e) to then plot (y, u) as a scatter-plot, labeling the dots 
              #         as the sbc. And nc is the number of bins for the histogram. 
	if(sum(comparison == c('Data', 'ECR', 'KL'))==1){
		r2 <- switch(comparison, 'Data' = 1, 'ECR' = 2, 'KL' = 3) 		
		perm <- as.matrix(read.table(file = paste(in_out$dir_res, 'perm.txt', sep =''), header = FALSE))[r2,]
		title <- paste(comparison, 'Scaled Components Predictive', sep ='')
	}else{
		perm <- seq(1, k, by = 1)
		title <- 'Scaled Components Predictive'
	}
	pred_comp <- as.matrix(read.table(file = paste(in_out$dir_res, pred_name, sep =''), header = FALSE))[,perm]
	class_probs <- as.matrix(read.table(file = paste(in_out$dir_res, prob_name, sep =''), header = FALSE))[,perm]	
	u <- runif(n, r$c, r$e)	
  for(i in 1:n){
    single_best[i] <- which.max(class_probs[i,])
  }
	hist(in_out$y, main= title, xlab = '', 
       ylab = 'Relative Frequency', xlim = c(r$a, r$b), ylim = c(r$c, r$d), probability = T, axes = F, 
       nclass = r$nc, col='gray80', border='white')
	axis(1, seq(r$a, r$b, length.out = 5), cex.axis = .8)
	axis(2, round(seq(0, r$d, length.out = 5), 3), cex.axis = .8)
  for(j in 1:k){
    lines(in_out$y_star, pred_comp[, j], main='', lwd = 2, col = j)
  }
  text(in_out$y, u, single_best, col = single_best, cex = 1.5)
	box(lwd = 2, col = 'black')
}
#------------------------------------------------------------#
#               14 - Plotting the Trace                      #
#------------------------------------------------------------#
#                IMPORTANT!!!!!!!!!                          #
#    The relabeling used must be generated first             #
#                                                            #
#               Input Data                                   #
#                                                            #
# k          - number of components                          #
# m          - number of iterations stored during the MCMC   #
# y          - data (univariate)                             #
# dir        - directory where the MCMC output and           #
#              relabeling should be stored.                  #
# label_name - the name that was given to the relabeling     #
# comparison - If this parameter is equal to:                #
#                   'Data', 'ECR' or 'KL'                    #
#              then the program will search for the file     #
#              perm.txt in the directory: dir, to order      #
#              the components according to a permutation.    #
#              This is only useful when comparing between    #
#              the different algorithms.                     #
#              For further details see the function:         #
#               9 - Comparison of Posterior Mean estimates   #
#              if no comparison is needed give a different   #
#              value.                                        #
#                                                            #
#              Output Data                                   #
#                                                            #
#  A graphic for the trace generated with a given relabeling #
#------------------------------------------------------------#
mytrace <- function(option = 'means', k, m, dir, label_name, comparison){
	r <- switch(option, 
							'weights' = paste(dir, 'w.txt', sep=''),  
							'means' = paste(dir, 'mu.txt', sep=''), 
							'variances' = paste(dir, 'sigma2.txt', sep=''))
	mm <- 10000
	I <- seq(m, m - (mm-1), by = -1) # to extract the last 10000 iterations
	if(sum(comparison == c('Data', 'ECR', 'KL'))==1){
		r2 <- switch(comparison, 'Data' = 1, 'ECR' = 2, 'KL' = 3) 		
		perm <- as.matrix(read.table(file = paste(in_out$dir_res, 'perm.txt', sep =''), header = FALSE))[r2,]
		title <- paste(comparison, 'Trace for the', option, sep = ' ')
	}else{
		perm <- seq(1, k, by = 1)
		title <- paste('Trace for the', option, sep = ' ')
	}
	label <- as.matrix(read.table(file= paste(dir, label_name, sep=''), header = FALSE))[I,]
	v <- as.matrix(read.table(file= r, header = FALSE))[I,]
	vt <- array(NA, c(mm, k))
	for(l in 1:mm){
		vt[l,] <- v[l, label[l, perm] + 1] 
	}
	cc <- quantile(as.vector(v), probs = 0.05, type = 8) 
	dd <- quantile(as.vector(v), probs = 0.95, type = 8)	
	plot(c(0, 10000), c(cc, dd), type = 'n', main = title, 
				axes = F, ylab = 'Trace', xlab = option, xlim = c(0, 10000), ylim = c(cc, dd))
	for(j in 1:k){
		points(vt[,j], pch = 46, col = j)
	}
	axis(1, seq(0, mm, length.out = 5), cex.axis=.8)
	axis(2, round(seq(cc, dd, length.out = 5), 1), cex.axis=.8)	
	box(lwd=2, col = 'black')
}
#------------------------------------------------------------#
#      Permutation to minimize the total relative error      #  
#------------------------------------------------------------#
#                                                            #
#  This function finds the permutation that minimizes the    #
#  total relative error between the posterior mean estimates #
#  and a reference value. This is useful to compare          #
#  posterior mean estimates "for the same components"        #  
#  calculated via different relabelings.                     # 
#  This function is used in function:                        #
#      9 - Comparison of Posterior Mean estimates            #
#                                                            #
#                   Input Data                               #
#                                                            #
# k           - number of components                         #
# ref_w       - reference or true values for the weights     #
# ref_mu      - reference or true values for the means       #
# ref_sigma2  - reference or true values for the variances   #
# w           - posterior mean estimate for the weights      #
# mu          - posterior mean estimate for the means        #
# sigma2      - posterior mean estimate for the variances    #
#                                                            #
#                   Output Data                              #
#                                                            #
#   - a permutation that minimizes the total relative error  #
#     between the posterior mean estimate and a reference    # 
#     value                                                  #  
#------------------------------------------------------------#
find_perm_min_tot_rel_error <- function(k, ref_w, ref_mu, ref_sigma2, w, mu, sigma2){
	label <- rep(0, k)
  .C("perm_min_rel_error", as.integer(k), as.double(ref_w), as.double(ref_mu), as.double(ref_sigma2), 
      as.double(w), as.double(mu), as.double(sigma2), as.integer(label))[[8]] + 1
}