#define _LARGEFILE_SOURCE
#define _LARGEFILE64_SOURCE
#define _FILE_OFFSET_BITS 64
#include <R.h>
#include "LSAP.h"

void KL_relabel(int *k, int *n, int *m, double *epsilon, int *cont, char **dir_label,
											char **dir_pz, char **dir_pz_fin){
  
  int **label, l, i, j, h, iterations, id;
  double **p, **C, q[*n][*k], q_tmp[*n][*k], min_C, L0, min_L0, C_real[*k][*k], diff;
  FILE *ffile_pz, *ffile_pz_fin, *ffile_label;
  
  ffile_label = fopen(*dir_label, "r+" );
  ffile_pz = fopen(*dir_pz, "r" );
  ffile_pz_fin = fopen(*dir_pz_fin, "w" );

  /* Allocating memory and initialising variables */
  id = 0; 
  label = Calloc(*m, int*);  
  p = Calloc((*n)*(*m), double *);
  label[0] = Calloc(*k, int);
  for(i = 0; i < *n; i++){
    p[id] = Calloc(*k, double);
    for(j = 0; j < *k; j++){
      q[i][j] = 0.0;
      q_tmp[i][j] = 0.0;      
    }
    id++;
  }  
  for(h = 1; h < *m; h++){
    label[h] = Calloc(*k, int);
    for(i = 0; i < *n; i++){
      p[id] = Calloc(*k, double);
      id++;
    }    
  }    
  C = Calloc(*k, double *);
  for(j = 0; j < *k; j++){
    C[j] = Calloc(*k, double);
  }
    
  /*-------------------------*/
  /* Calculating the first q */
  /*-------------------------*/
  id = 0;
  for(h = 0; h < *m; h++){
    for(j = 0; j < *k; j++){
      if(fscanf(ffile_label, "%d ", &label[h][j]) != 1 ){
				Rprintf("Problem reading label first time, iteration %d\n", h);
				exit(1);
      }      
    }
    for(i = 0; i < *n; i++){
      for(j = 0; j < *k; j++){
				if(fscanf(ffile_pz, "%lf ", &p[id][j]) != 1 ){
					Rprintf("Problem reading pz first time\n");
					exit(1);
				}
      }
      for(j = 0; j < *k; j++){
				q[i][j] += p[id][label[h][j]];	
      }
      id++;
    }
  }    
  for(i = 0; i < *n; i++){
    for(j = 0; j < *k; j++){
      q[i][j] /= ((double) *m);
    }
  }
  rewind(ffile_label); /* To store the labels at the end in the same file */

  iterations = 0;
  L0 = 0.0;
  min_L0 = 0.0;
  diff = *epsilon + 1.0;
  /*----------------------*/
  /*  Starting big loop   */
  /*----------------------*/
  while(diff > *epsilon && iterations < *cont){
    iterations++;
    min_L0 = L0;
    L0 = 0.0;
    id = 0;
    
    for(h = 0; h < *m; h++){
      
      min_C = 0.0;
      for(j = 0; j < *k; j++){
				for(l = 0; l < *k; l++){
					C[j][l] = 0.0;
				}
      }

      /* Calculating the cost matrix C */
      for(l = 0; l < *k; l++){
				for(j = 0; j < *k; j++){
					for(i = 0; i < *n; i++){
						if(p[id][l] > 0.0){
							if(q[i][j] > 0.0){
								C[j][l] += p[id][l]*(log(p[id][l]) - log(q[i][j]));
							}else{
								C[j][l] += 710.0; /* - log(q) = Inf => 710.0 = log(exp(710.0) = Inf) */
							}
						}	    
						/* If there are negative values => "re-scale" the matrix */
						if(C[j][l] < min_C){
							min_C = C[j][l];
						}
						id++;
					}
					C_real[j][l] = C[j][l]; /* the real Cost matrix*/
					id -= *n; 
				}
      }

      if(0.0 > min_C){	
			/* Rescaling C to do not have negative values */
				for(j = 0; j < *k; j++){
					for(l = 0; l < *k; l++){
						C[j][l] -= min_C;	    
					}
				}
      }      
      /*Solving the assignment problem => Hungarian method: from R (library clue)*/
      solve_LSAP(C, *k, label[h]);

      /* Calculating the value of the function to minimise */
      for(j = 0; j < *k; j++){
				L0 += C_real[j][label[h][j]];
      }
      /*  Calculating the new q => q_tmp */
      for(i = 0; i < *n; i++){
				for(j = 0; j < *k; j++){
					q_tmp[i][j] += p[id][label[h][j]];
				}
				id++;
      }
          
    }   /* closing the for loop */
    diff = abs(min_L0 - L0);  
    Rprintf("Iteration %d, value of the function to minimise %lg\n", iterations, L0);

    /* Updating q's '*/
    for(i = 0; i < *n; i++){
      for(j = 0; j < *k; j++){
				q_tmp[i][j] /= ((double) *m);
				q[i][j] = q_tmp[i][j];
				q_tmp[i][j] = 0.0;	
      }
    }
  }   /* closing the while loop */
  
  Rprintf("Fix point at iteration %d, minimum value achieved %lg\n", iterations, min_L0);

  for(h = 0; h < *m; h++){
    for(j = 0; j < *k; j++){
      fprintf(ffile_label, "%d \t", label[h][j]);
    }
    fprintf(ffile_label, "\n");
  }
  /* storing the allocation probabilities */
  for(i = 0; i < *n; i++){
    for(j = 0; j < *k; j++){
      fprintf(ffile_pz_fin, "%12.16f \t", q[i][j]);
    }
    fprintf(ffile_pz_fin, "\n");
  }

  /* Releasing memory */
  id = 0;
  for(h = 0; h < *m; h++){
    Free(label[h]);
    for(i = 0; i < *n; i++){
      Free(p[id]);
      id++;
    }
  }
  Free(label);
  Free(p);
  for(j = 0; j < *k; j++){
    Free(C[j]);
  }
  Free(C);    
  fclose(ffile_label);
  fclose(ffile_pz);
  fclose(ffile_pz_fin);
}